@@ -43,6 +43,10 @@ class Curve(ABC):
     def line_representation(self) -> List:
         pass
 
+    @abstractmethod
+    def force_continuity(self, next_point: Vector2) -> Vector2:
+        pass
+
     @classmethod
     def orientation(cls, origin, destination) -> List[Orientation]:
         orientation = None
@@ -200,6 +204,19 @@ class Arc(Curve):
     def length(self):
         return (self.arc_rad_end - self.arc_rad_start) * (self.origin - self.mid_point).length()
 
+    def force_continuity(self, next_point: Vector2) -> Vector2:
+        end_point = self.end_point()
+        if self.orientation in (Orientation.UP_LEFT, Orientation.DOWN_LEFT) and next_point.x > end_point.x:
+            return Vector2(end_point.x, next_point.y)
+        elif self.orientation in (Orientation.UP_RIGHT, Orientation.DOWN_RIGHT) and next_point.x < end_point.x:
+            return Vector2(end_point.x, next_point.y)
+        elif self.orientation in (Orientation.RIGHT_UP, Orientation.LEFT_UP) and next_point.y > end_point.y:
+            return Vector2(next_point.x, end_point.y)
+        elif self.orientation in (Orientation.RIGHT_DOWN, Orientation.LEFT_DOWN) and next_point.y < end_point.y:
+            return Vector2(next_point.x, end_point.y)
+        else:
+            return next_point
+
 
 
 class Line(Curve):
@@ -223,6 +240,9 @@ class Line(Curve):
                   f"{e}")
             raise
 
+    def force_continuity(self, next_point: Vector2) -> Vector2:
+        return next_point
+
 class CubicBezier(Curve):
     def __init__(self, id:str, control_points: List[Vector2]):
         Curve.__init__(self, id, control_points[0])
@@ -311,6 +331,105 @@ class CubicBezier(Curve):
         return result
 
 
+    def force_continuity(self, next_point: Vector2) -> Vector2:
+        return next_point.project_onto(self.control_points[-1], self.control_points[-2])
+
+class CatmullRom(Curve):
+    def __init__(self, id:str, control_points: List[Vector2]):
+        Curve.__init__(self, id, control_points[0])
+
+        if len(control_points) < 3:
+            raise ValueError(f"Invalid input for control points. Must have at least 3 values but list of lenght {len(control_points)} was provided: [{control_points}]")
+        self.control_points = control_points
+
+    def line_representation(self):
+        return self.compute_curve_lines()
+
+    def end_point(self):
+        return self.control_points[-1]
+
+    def compute_curve_lines(self):
+        b_points = self.compute_bezier_points([(x.x, x.y) for x in self.control_points])
+        ret = []
+        for ii in range(0, len(b_points)):
+            if ii == 0:
+                continue
+            else:
+                start = Vector2(b_points[ii - 1][0], b_points[ii - 1][1])
+                end = Vector2(b_points[ii][0], b_points[ii][1])
+                ret.append(Line(self.id + f"_{ii}", start, end))
+        return ret
+
+    def compute_bezier_points(self, vertices, numPoints=None):
+        if numPoints is None:
+            numPoints = 30
+        if numPoints < 2 or len(vertices) != 3:
+            return []
+
+        result = []
+
+
+
+
+        b0x = vertices[0][0]
+        b0y = vertices[0][1]
+        b1x = vertices[1][0]
+        b1y = vertices[1][1]
+        b2x = vertices[2][0]
+        b2y = vertices[2][1]
+        b3x = vertices[3][0]
+        b3y = vertices[3][1]
+
+        # Compute polynomial coefficients from Bezier points
+        ax = -b0x + 3 * b1x + -3 * b2x + b3x
+        ay = -b0y + 3 * b1y + -3 * b2y + b3y
+
+        bx = 3 * b0x + -6 * b1x + 3 * b2x
+        by = 3 * b0y + -6 * b1y + 3 * b2y
+
+        cx = -3 * b0x + 3 * b1x
+        cy = -3 * b0y + 3 * b1y
+
+        dx = b0x
+        dy = b0y
+
+        # Set up the number of steps and step size
+        numSteps = numPoints - 1  # arbitrary choice
+        h = 1.0 / numSteps  # compute our step size
+
+        # Compute forward differences from Bezier points and "h"
+        pointX = dx
+        pointY = dy
+
+        firstFDX = ax * (h * h * h) + bx * (h * h) + cx * h
+        firstFDY = ay * (h * h * h) + by * (h * h) + cy * h
+
+        secondFDX = 6 * ax * (h * h * h) + 2 * bx * (h * h)
+        secondFDY = 6 * ay * (h * h * h) + 2 * by * (h * h)
+
+        thirdFDX = 6 * ax * (h * h * h)
+        thirdFDY = 6 * ay * (h * h * h)
+
+        # Compute points at each step
+        result.append((int(pointX), int(pointY)))
+
+        for i in range(numSteps):
+            pointX += firstFDX
+            pointY += firstFDY
+
+            firstFDX += secondFDX
+            firstFDY += secondFDY
+
+            secondFDX += thirdFDX
+            secondFDY += thirdFDY
+
+            result.append((int(pointX), int(pointY)))
+
+        return result
+
+    def force_continuity(self, next_point: Vector2) -> Vector2:
+        return next_point
+
 class CurveType(Enum):
     ARC = 1,
     CUBICBEZIER = 2,
@@ -403,7 +522,7 @@ class CurveBuilder:
             curves.append(CubicBezier(self.id_provider(), points[ii:ii+4]))
         return curves
 
-    def curves_from_temp_next_points(self, next_points: List[Vector2]):
+    def curves_from_temp_next_points(self, next_points: List[Vector2], force_continuity: bool=True):
         all_points = self.current_points + next_points
         points = []
         for i in all_points:
@@ -412,4 +531,9 @@ class CurveBuilder:
 
         return self.curves_from_points(self.current_curve_type.value, points)
 
+    def force_continuity(self, next_point: Vector2):
+        curves = self.curves_from_points(self.current_curve_type.value, self.current_points)
+        return curves[-1].force_continuity(next_point)
+
+
 
